/* 
 * File:   main_full_system.c
 * Author: Sam Blanchet
 *
 * Created on February 14, 2018, 2:38 PM
 */

//Configuration Bits
//******************************************************************************
// DEVCFG3
// USERID = No Setting
#pragma config PMDL1WAY = OFF            // Peripheral Module Disable Configuration (Allow only one reconfiguration)
#pragma config IOL1WAY = OFF             // Peripheral Pin Select Configuration (Allow only one reconfiguration)
#pragma config FUSBIDIO = OFF            // USB USID Selection (Controlled by the USB Module)
#pragma config FVBUSONIO = OFF           // USB VBUS ON Selection (Controlled by USB Module)

// DEVCFG2
#pragma config FPLLIDIV = DIV_2         // PLL Input Divider (2x Divider)
#pragma config FPLLMUL = MUL_18         // PLL Multiplier (20x Multiplier)
#pragma config UPLLIDIV = DIV_12        // USB PLL Input Divider (12x Divider)
#pragma config UPLLEN = OFF             // USB PLL Enable (Disabled and Bypassed)
#pragma config FPLLODIV = DIV_2         // System PLL Output Clock Divider (PLL Divide by 1)

// DEVCFG1
#pragma config FNOSC = PRIPLL           // Oscillator Selection Bits (Primary Osc w/PLL (XT+,HS+,EC+PLL))
#pragma config POSCMOD = HS
#pragma config FPBDIV = DIV_1           // Peripheral Clock Divisor (Pb_Clk is Sys_Clk/1)

#pragma config FSOSCEN = OFF            // Secondary Oscillator Enable (Disabled)
#pragma config IESO = ON                // Internal/External Switch Over (Enabled)
#pragma config OSCIOFNC = OFF           // CLKO Output Signal Active on the OSCO Pin (Disabled)
#pragma config FCKSM = CSDCMD           // Clock Switching and Monitor Selection (Clock Switch Disable, FSCM Disabled)
#pragma config WDTPS = PS1048576        // Watchdog Timer Postscaler (1:1048576)
#pragma config WINDIS = OFF             // Watchdog Timer Window Enable (Watchdog Timer is in Non-Window Mode)
#pragma config FWDTEN = OFF             // Watchdog Timer Enable (WDT Disabled (SWDTEN Bit Controls))
#pragma config FWDTWINSZ = WINSZ_25     // Watchdog Timer Window Size (Window Size is 25%)

// DEVCFG0
#pragma config JTAGEN = OFF             // JTAG Enable (JTAG Disabled)
#pragma config ICESEL = ICS_PGx1        // ICE/ICD Comm Channel Select (Communicate on PGEC1/PGED1)
#pragma config PWP = OFF                // Program Flash Write Protect (Disable)
#pragma config BWP = OFF                // Boot Flash Write Protect bit (Protection Disabled)
#pragma config CP = OFF                 // Code Protect (Protection Disabled)
#pragma config DEBUG = ON               // Background Debugger Enable
//*****************************************************************************

#define _SUPPRESS_PLIB_WARNING
//#define PPSUnLock __builtin_write_OSCCONL(OSCCON & 0xbf)

#include <./proc/p32mx270f256b.h>
#include <./lega-c/plib.h>
#include <./peripheral/osc.h>
#include <./peripheral/system.h>
#include <xc.h>
#include "ff.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "SDlib16_delay.h"

//Define LEDs
#define Lrecord LATBbits.LATB5 //recording LED
#define LrecordIO TRISBbits.TRISB5

//Define pins
#define recordPinStatus PORTBbits.RB7 //button pin which is idle high
#define recordPinIO TRISBbits.TRISB7 //button pin I/O

//Define GPIO Toggles (U1TX pin)
//#define GPIOtoggle LATBbits.LATB4 = 1; _nop(); LATBbits.LATB4 = 0;
//#define GPIOtoggleIO TRISBbits.TRISB4

//Define variables
#define debounceCounts      2           //20ms debounce
#define syncCounts          3000        //30s sync time
#define ledOffCounts        10          //100ms flash
#define GetSystemClock()    (80000000L)
#define GetPBClock()        (40000000L)
#define UARTBaudRate        50000
#define blockWriteSize      16384        //size to be written to SD card
#define dataByteSize        1           //how many bytes per ADC entry
                                        //CHANGE ADC OUTPUT ARRAY TYPE IF YOU CHANGE THIS
#define GAIN1 PORTBbits.RB3
#define GAIN2 PORTBbits.RB8
#define GAIN3 PORTBbits.RB9


//Global variables
int dataMarker=0;
int pinState=1; //idle high
int syncCounter=syncCounts;
int debounceCounter=0;
int ledOffCounter=0;
int record=0;
unsigned char ADCOutput1[blockWriteSize/dataByteSize];
unsigned char ADCOutput2[blockWriteSize/dataByteSize];
unsigned char *pADCOutput;
int ADCOutputIndex = 0;
int ADCcount = 0;           //DEBUGGING

//Work registers for fs command
DWORD acc_size;			/* Work register for fs command */
WORD acc_files, acc_dirs;
FILINFO Finfo;
const BYTE ft[] = {0,12,16,32};

//File system object
FATFS Fatfs;
FATFS *fs;				/* Pointer to file system object */
FIL file1, file2;		/* File objects */

//Function Prototypes
unsigned char AD_int(void);

int main (void)
{
    

//Set Up timers
    //Configure timer 3
    T3CONbits.ON=0;
    T3CONbits.TCS = 0;       //PBCLK source
    T3CONbits.TCKPS = 0b011; //40 MHz/8 = 5MHz
    TMR3 = 0;
    PR3 = 50000; //makes this a 10mS interrupt - 100Hz
   
//Set Up Interrupts
    INTCONbits.MVEC = 1; //enable multi-vector interrupts
    //ADC
    IEC0bits.AD1IE = 1;  //enable ADC interrupt
    IPC5bits.AD1IP = 5;  //priority 5
    //Timer 1 - Button press/debouncing
    IFS0bits.T3IF = 0; //clear interrupt flag
    IPC3bits.T3IP = 3; //priority 3
    IEC0bits.T3IE = 1; //enable timer 1 interrupt
  
//Turn off analog pins 
    ANSELB=0;
    
//Initialize ADC
    AD_int();
    
//Setup the UART and SPI pins
   // PPSUnLock();
    SDI2R = 0b0011; //MISO is B13
    RPB11R = 0b0100; //MOSI is B11
    RPB4R = 0b0001; //UART1 Tx on pin B4
    U1RXR = 0b0010; //UART1 Rx on pin A4
   // PPSLock();

    
    // check DIP switches to determine appropriate gin control
    CNPUBbits.CNPUB3 = 1;
    CNPUBbits.CNPUB8 = 1;
    CNPUBbits.CNPUB9 = 1;

   int gain1, gain2, gain3;
   
   gain1 = GAIN1;
   gain2 = GAIN2;
   gain3 = GAIN3;
   
    TRISBbits.TRISB3 = 1;
    TRISBbits.TRISB8 = 1;
    TRISBbits.TRISB9 = 1;
   
    int CVR = 4;
    int CVRR = 1;

        
    if (gain1 == 0 && gain2 == 0 && gain3 == 0){  //Gain 5.6 Voltage 1.03
        CVRR = 0;
        CVR = 2;
    }
    else if (gain1 == 1 && gain2 == 0 && gain3 == 0){  //Gain 11.2 Voltage .96
        CVRR = 1;
        CVR = 7;
    }
    else if (gain1 == 0 && gain2 == 1 && gain3 == 0){  //Gain 13.6 Voltage .93
        CVRR = 0;
        CVR = 1;
    }    
    else if (gain1 == 0 && gain2 == 0 && gain3 == 1){  //Gain 21.6  Voltage .83
        CVRR = 0;
        CVR = 0;
    }   
    else if (gain1 == 1 && gain2 == 1 && gain3 == 0){ // Gain 32.8  Voltage .69
        CVRR = 1;
        CVR = 5;
    }
    else if (gain1 == 1 && gain2 == 0 && gain3 == 1){ //Gain 44  Voltage .55
        CVRR = 1;
        CVR = 4;
    }    
    else if (gain1 == 0 && gain2 == 1 && gain3 == 1){  // Gain 55.2  Voltage .41
        CVRR = 1;
        CVR = 3;
    }    
    else if (gain1 == 1 && gain2 == 1 && gain3 == 1){  //Gain 65.6  Voltage .28
        CVRR = 1;
        CVR = 2;
    }

    TRISBbits.TRISB14 = 0;
    CVRCONbits.CVROE = 1;
    CVRCONbits.CVRR = CVRR;
    CVRCONbits.CVR = CVR;
    CVRCONbits.ON = 1;  
    
//Set LEDs&Toggle as outputs and initialize as off
    LrecordIO=0;
    Lrecord=0;
//    GPIOtoggleIO=0;
    
//Set button as input
    recordPinIO=1;

//Turn on LED
    delay_ms(100);
    Lrecord=1;

//Delay for Startup
    delay_ms(300);

    Lrecord=0;

    delay_ms(200);
    
//Start Timer 3 for button checking
    __builtin_enable_interrupts(); //turn on interrupts
    T3CONbits.ON=1;
//----------------------------------------------------------------
    
//Begin recording/file creation loop
    while(1){
        
        //Wait for start button to be pressed
        while(!record);  //wait for button press
        Lrecord=1;
        
    //Initialize Disk
        disk_initialize(0);

    //Mount File system
        f_mount(0, &Fatfs);

    //Open/create Fileset.dex
        FRESULT FOpenRes;
        long accessmode;
        long accessmode2;
        char file1name[]= "Fileset.dex";
        char file2name[]= "NBII000.dat";
        accessmode2= (FA_CREATE_ALWAYS | FA_OPEN_ALWAYS | FA_CREATE_NEW | FA_WRITE |FA_READ);
        accessmode= (FA_OPEN_ALWAYS | FA_WRITE |FA_READ);
        FOpenRes=f_open(&file1, file1name, (BYTE)accessmode);

    //Read 1 byte from file
        char Buff=0;
        unsigned int numread=0;

        if(FOpenRes==FR_OK)
            {
                FOpenRes=f_read (&file1, &Buff, 1, &numread);
            }
        else
            {
                //Create new file
                f_close(&file1);
                FOpenRes=f_open(&file1, file1name, (BYTE)accessmode2);
                numread=0;
            }

        if (numread==0)
            {//no bytes read, new fileset
                Buff=0;
                FOpenRes=f_write(&file1, &Buff, 1, &numread);
                //numread recycled as numwritten
            }
            else
            { //Overwrite first byte
                f_close(&file1);
                f_open(&file1,file1name, (BYTE)accessmode);
                Buff++;
                FOpenRes=f_write(&file1, &Buff, 1, &numread);
            }

    //close first file
        f_close(&file1);


    //Determine name of new file to create based on index number
        if (Buff<=9)
            {file2name[6]=Buff+48;}
        else if (Buff<=99)
            {file2name[5]=Buff/10 +48;
             file2name[6]=Buff%10 +48;
            }

    //Recycle the pointer and the access mode
        FOpenRes=f_open(&file1, file2name, (BYTE)accessmode);

    //Turn on timer and ADC
        IFS0bits.AD1IF=0;   //clear ADC flag before turning on
        AD1CON1bits.ON = 1; //turn on ADC system to start recording data

    //Collect data until button pressed again
        pADCOutput = ADCOutput1;
        while(record){
            //reset the index so we start to rewrite the array next interrupt
            //write the 4k of data which should still be in tact
            //the data is set in the f_write function so we can rewrite ADCOutput now
            //each interrupt changes ADCOutput according to ADCOutputIndex

            while(ADCOutputIndex<blockWriteSize/dataByteSize);
            pADCOutput = ADCOutput2;
            ADCOutputIndex = 0;
            FOpenRes=f_write(&file1, ADCOutput1, blockWriteSize, &numread);
           // f_sync(&file1);
            
            while(ADCOutputIndex<blockWriteSize/dataByteSize);
            pADCOutput = ADCOutput1;
            ADCOutputIndex = 0;
            FOpenRes=f_write(&file1, ADCOutput2, blockWriteSize, &numread);
            //f_sync(&file1);
        }
    //Turn off ADC, LED, and sync timer
        AD1CON1bits.ON = 0;
        Lrecord=0;
        T2CONbits.ON=0;
    //close second file
        f_close(&file1);

    //unmount file system
        f_mount(0,NULL);
    //Flash LED twice to signal we are done closing file
        delay_ms(100);
        Lrecord=1;
        delay_ms(100);
        Lrecord=0;
        delay_ms(100);
        Lrecord=1;
        delay_ms(100);
        Lrecord=0;
     
    
    //Return to top and wait for record button press again
    }

    return EXIT_FAILURE; //we should never get here
}

/*
 * AD system initialization function
 * 
 * Initialize to sample at 200kHz with 2 8-length buffers
 * Pin used is AN5 aka RB3
 */
unsigned char AD_int(void) {
    
    //Set digital pins to analog   
    //WE ARE USING AN5 = B3
    ANSELAbits.ANSA0 = 1; //New value: A0
    TRISAbits.TRISA0 = 1;//New value: A0
    
     ANSELAbits.ANSA1 = 1; //New value: A1
    TRISAbits.TRISA1 = 1;//New value: A1
    
    AD1CON3bits.ADRC = 0;       // Use PB clock
    AD1CON1bits.SSRC = 0b111; // auto-convert
    AD1CON2bits.VCFG = 0b000; // AVdd+/AVss
    AD1CON2bits.SMPI = 0b0111; // set flag every 8 conversions
    AD1CON2bits.BUFM = 0b1; // dual-mode
    AD1CON1bits.FORM = 0b000; // 16-bit integer
    AD1CON1bits.ASAM = 0b1; // auto-sample
    AD1CON3bits.SAMC = 6; //Auto-sample time bits of hold time
    AD1CON3bits.ADCS = 2; //ADC conversion clock select bits, should be 1
  // AD1CHSbits.CH0SA = 0; // use pin AN5. New value: 0
    
    AD1CON2bits.CSCNA = 1;      // Set to scan inputs on MUX A
    AD1CON2bits.ALTS=0;         // only use MUX A
    AD1CSSL = 0b11;         //scan back and forth between AN0 and AN1
    return EXIT_SUCCESS;
}

/*
 * ADC System Interrupt Service
 * 
 * Take the appropriate buffer data to the global array
 * Add data marker as necessary
 */
void __ISR(23, IPL5AUTO) ADCinterrupt(void)
{
    //Which size 8 buffer to use?
    if (AD1CON2bits.BUFS==0) {
       
        pADCOutput[ADCOutputIndex++] = (ADC1BUF8 >> 2);
        pADCOutput[ADCOutputIndex++] = (ADC1BUF9 >> 2);
        pADCOutput[ADCOutputIndex++] = (ADC1BUFA >> 2);
        pADCOutput[ADCOutputIndex++] = (ADC1BUFB >> 2);
        pADCOutput[ADCOutputIndex++] = (ADC1BUFC >> 2);
        pADCOutput[ADCOutputIndex++] = (ADC1BUFD >> 2);
        pADCOutput[ADCOutputIndex++] = (ADC1BUFE >> 2);
        pADCOutput[ADCOutputIndex++] = (ADC1BUFF >> 2);
        IFS0bits.AD1IF=0; 
        //clear flag
    } else if (AD1CON2bits.BUFS==1) { 
        
        pADCOutput[ADCOutputIndex++] = (ADC1BUF0 >> 2);
        pADCOutput[ADCOutputIndex++] = (ADC1BUF1 >> 2);
        pADCOutput[ADCOutputIndex++] = (ADC1BUF2 >> 2);
        pADCOutput[ADCOutputIndex++] = (ADC1BUF3 >> 2);
        pADCOutput[ADCOutputIndex++] = (ADC1BUF4 >> 2);
        pADCOutput[ADCOutputIndex++] = (ADC1BUF5 >> 2);
        pADCOutput[ADCOutputIndex++] = (ADC1BUF6 >> 2);
        pADCOutput[ADCOutputIndex++] = (ADC1BUF7 >> 2); 
        IFS0bits.AD1IF=0;                       //clear flag
    }
    
}

/*
 * Timer 3 Interrupt Service
 * 
 * 100 Hz timer interrupt that checks the button
 * Decrement a counter each time interrupt happens and pin is pulled high
 * Once counter reaches zero treat it as button=pressed
 * For each input you maintain a counter. You also store the previous state 
 * of the port. Whenever the input has changed since the last interrupt, 
 * you load the corresponding counter with the debounce time you want 
 * (start with 30ms) in terms of the number of interrupt periods. 
 * You decrement any non-zero counter and act on any which reach zero.
 * 
 */
void __ISR(12, IPL3AUTO) Timer3Interrupt(void)
{
    IFS0bits.T3IF=0;    //clear flag
    
   // f_sync(&file1);

    if(recordPinStatus!=pinState){ //if pin state has changed from last time
        pinState=recordPinStatus; //reassign value
        debounceCounter=debounceCounts; //load debounce counter
    }
    if(debounceCounter>0&&pinState==0){
        debounceCounter--;   //Decrement counter
        if(debounceCounter==0){     //Button treated as pressed after 30ms
            if(record==0){          //start recording
                record=1;
            } else if(record==1){   //stop recording
                record=0;
            }        
        }
    }
    return;
}

